@@ -1,5 +1,4 @@
 //  
-// $Id$
 //  Initial port performed by Greg Onufer (exodus@cheers.bungi.com)
 //
 #ifndef _CPUMETER_H_
@@ -12,7 +11,7 @@
 
 class CPUMeter : public FieldMeterGraph {
  public:
-	CPUMeter(XOSView *parent, kstat_ctl_t *kcp, const int cpuid = 0);
+	CPUMeter(XOSView *parent, kstat_ctl_t *kcp, int cpuid = 0);
 	~CPUMeter(void);
 
 	const char *name(void) const { return "CPUMeter"; }
